@@ -8,8 +8,7 @@ import unidecode
 import warnings
 
 from pandas.api.types import is_float_dtype, is_numeric_dtype, is_string_dtype
-#from humailib.cloud_tools import GoogleBigQuery, GoogleCloudStorage
-
+from humailib.cloud_tools import GoogleBigQuery, GoogleCloudStorage, AzureBlobStorage, AzureDataLake
 
 def instantiate_class(module_name, class_name):
     module = __import__(module_name, globals(), locals(), [class_name])
@@ -18,6 +17,37 @@ def instantiate_class(module_name, class_name):
     return class_
 
 
+keys = {
+    'azure' : {
+        'gcs' : 'DefaultEndpointsProtocol=https;AccountName=humaimain;AccountKey=VSWuKEKDm3FC9dV43g6giU2RD93XkMyZ4OrXiBqOdu/nHozxlZjTmZcOyg7D2LzI/qjGSvanMrouG9fzhDOPVw==;EndpointSuffix=core.windows.net',
+        'gbq_humai_sb' : 'DefaultEndpointsProtocol=https;AccountName=humaiadlsgen2;AccountKey=1awKbHfeHcqWYtGut/t2t8dBM0ZLXCWlyVUePE+EuqXOzAXeDwF8AOb9MvHvkexuPNDmfAJt8iab80Xp97lM9Q==;EndpointSuffix=core.windows.net',
+        'gbq_humai_insights' : 'DefaultEndpointsProtocol=https;AccountName=humaiadlsgen2;AccountKey=1awKbHfeHcqWYtGut/t2t8dBM0ZLXCWlyVUePE+EuqXOzAXeDwF8AOb9MvHvkexuPNDmfAJt8iab80Xp97lM9Q==;EndpointSuffix=core.windows.net',
+    },
+    'gcp' : {
+        'gcs' : 'service_account_keys/humai-sb-7b9db70de787.json',
+        'gbq_humai_sb' : 'service_account_keys/humai-sb-7b9db70de787.json',
+        'gbq_humai_insights' : 'service_account_keys/humai-insights-78e122a483b7.json',
+    }
+}
+
+def get_io(platform, path='../../'):
+
+    if platform.upper() == 'AZURE':
+        print("[get_io] Microsoft Azure...")
+        key = keys['azure']
+        gcs = AzureBlobStorage(key=key['gcs'])
+        gbq_humai_sb = AzureDataLake(project='humai-sb', key=key['gbq_humai_sb'])
+        gbq_humai_insights = AzureDataLake(project='humai-insights', key=key['gbq_humai_insights'])
+    else:
+        print("[get_io] Google Cloud Platform...")
+        key = keys['gcp']
+        gcs = GoogleCloudStorage(key=path+key['gcs'])
+        gbq_humai_sb = GoogleBigQuery(project='humai-sb', key=path+key['gbq_humai_sb'])
+        gbq_humai_insights = GoogleBigQuery(project='humai-insights', key=path+key['gbq_humai_insights'])
+
+    return gcs, gbq_humai_sb, gbq_humai_insights
+
+
 """
 Column type definitions from the data dictionary.
     
@@ -219,10 +249,10 @@ def load_table(
         convert_table_columns(df, table_type=table_type, datetime_format=datetime_format)    
 
         if table_type == 'transactions' and keep_most_recent_trans_id:
-            print("[load_and_merge_tables] Keeping only the most recent Transaction ID, if there's multiples...")
-            print("[load_and_merge_tables]   {:,} rows total.".format(len(df)))
+            print("[load_table] Keeping only the most recent Transaction ID, if there's multiples...")
+            print("[load_table]   {:,} rows total.".format(len(df)))
             keep_most_recent_transaction_id(df)
-            print("[load_and_merge_tables]   {:,} rows left.".format(len(df)))
+            print("[load_table]   {:,} rows left.".format(len(df)))
 
     return df
 
@@ -270,10 +300,10 @@ def load_csv(
         convert_table_columns(df, table_type=table_type, datetime_format=datetime_format)
     
         if table_type == 'transactions' and keep_most_recent_trans_id:
-            print("[load_and_merge_tables] Keeping only the most recent Transaction ID, if there's multiples...")
-            print("[load_and_merge_tables]   {:,} rows total.".format(len(df)))
+            print("[load_csv] Keeping only the most recent Transaction ID, if there's multiples...")
+            print("[load_csv]   {:,} rows total.".format(len(df)))
             keep_most_recent_transaction_id(df)
-            print("[load_and_merge_tables]   {:,} rows left.".format(len(df)))
+            print("[load_csv]   {:,} rows left.".format(len(df)))
 
     return df
 
@@ -298,22 +328,25 @@ def load_and_merge_tables(
     
     print("[load_and_merge_tables] Loading tables...")
     df = None
-    for date_range in date_ranges:
-        
-        if table_name is not None:
-            table = "{}.{}_{}".format(dataset, table_name, date_range)
-        else:
-            table = "{}.{}_{}".format(dataset, table_type, date_range)
-
-        df_tmp = gbq.download_table_to_pandas(
-            table,
-            flush_cache=flush_cache
-        )
-        if df_tmp is not None:
-            if df is None:
-                df = df_tmp.copy()
+    if (date_ranges is None) or (len(date_ranges) > 0):
+        for date_range in date_ranges:
+            
+            if table_name is not None:
+                table = "{}.{}_{}".format(dataset, table_name, date_range)
             else:
-                df = df.append(df_tmp, sort=True, ignore_index=True)
+                table = "{}.{}_{}".format(dataset, table_type, date_range)
+
+            df_tmp = gbq.download_table_to_pandas(
+                table,
+                flush_cache=flush_cache
+            )
+            if df_tmp is not None:
+                if df is None:
+                    df = df_tmp.copy()
+                else:
+                    df = df.append(df_tmp, sort=True, ignore_index=True)
+    else:
+        df = gbq.download_table_to_pandas("{}.{}".format(dataset, table_name))
 
     if df is None:
         return None
@@ -371,16 +404,22 @@ def upload_and_append_table_unique(
         flush_cache=flush_cache,
     )
     
-    print(f"Size of existing table before appending: {len(df_existing)}")
-    df_existing = df_existing.append(df, sort=True, ignore_index=True)
-    df_existing.drop_duplicates(subset=unique_subset, keep='first', inplace=True)
-    print(f"Size of table after appending and removing duplicates: {len(df_existing)}")
+    if df_existing is not None:
     
-    gbq.upload_and_replace_table(
-        df_existing,
-        dataset_table,
-    )
+        print(f"Size of existing table before appending: {len(df_existing):,}")
+        df_existing = df_existing.append(df, sort=True, ignore_index=True)
+        df_existing.drop_duplicates(subset=unique_subset, keep='first', inplace=True)
+        print(f"Size of table after appending and removing duplicates: {len(df_existing):,}")
     
+        gbq.upload_and_replace_table(
+            df_existing,
+            dataset_table,
+        )
+    else:
+        gbq.upload_and_replace_table(
+            df,
+            dataset_table,
+        )        
 
 def merge_dataframes(dataframes, on, how='left', overwrite_cols=False):
     
@@ -457,7 +496,8 @@ def gbq_friendly_column_name(column):
     col_name = column
     if str.isdigit(col_name[0]):
         col_name = '_' + col_name
-        
+
+    col_name = re.sub(r'[À-ÖØ-öø-ÿ]', '_', col_name)
     return unidecode.unidecode(re.sub(r'[^\w]', '_', col_name).replace('-','_')).replace('+','plus').replace('-','to')
 
 
